# SPDX-FileCopyrightText: ☭ Emery Hemingway
# SPDX-License-Identifier: Unlicense

include ./protocol

from std/strutils import normalize
import syndicate, syndicate/relays

{.passC: staticExec("pkg-config --cflags libnotify").}
{.passL: staticExec("pkg-config --libs libnotify").}

const notifyHeader = "<libnotify/notify.h>"
{.pragma: notify, header: notifyHeader, importc: "notify_$1".}
{.pragma: notification, header: notifyHeader, importc: "notify_notification_$1".}

type
  Notification {.importc: "NotifyNotification".} = ptr object
    discard
  GError {.importc, header: "glib/gerror.h".} = object
    message: cstring

proc g_free(mem: pointer) {.importc.}
proc g_error_free(err: ptr GError) {.importc, header: "glib/gerror.h".}

proc init(appName: cstring): bool {.notify.}
proc uninit() {.notify.}

proc is_initted: bool {.notify.}

proc get_server_info(name, vendor, version, specVersion: ptr (ptr char)): bool {.notify.}

proc notification_new(summary, body: cstring; icon: cstring = nil): Notification {.notify.}

proc set_app_name(n: Notification; s: cstring) {.notification.}
proc set_category(n: Notification; s: cstring) {.notification.}

type NotifyUrgency {.importc.} = enum Low, Normal, Critical
proc set_urgency(n: Notification; u: NotifyUrgency) {.notification.}

proc show(n: Notification) =
  proc show(n: Notification; err: ptr (ptr GError)): bool {.notification.}
  var err: ptr GError
  if not show(n, addr err):
    doAssert not err.isNil, "GLIB shit itself without setting an error"
    doAssert not err.message.isNil, "GLIB shit itself without setting an error message"
    stderr.writeLine "show failed: ", err.message
  if not err.isNil:
    g_error_free(err)

proc assertServerInfo(turn: var Turn; ds: Cap) =
  var name, vendor, version, specVersion: cstring
  if get_server_info(nil, nil, nil, nil):
    discard
  else:
    stderr.writeLine "failed to get server info!"

type Args {.preservesDictionary.} = object
  dataspace: Cap

runActor("libnotify_actor") do (ds: Cap; turn: var Turn):
  connectStdio(turn, ds)
  during(turn, ds, dropType Args) do:
    doAssert init("libnotify_actor"), "failed to initialize libnotify"

    during(turn, ds, ?Args) do (ds: Cap):
      assertServerInfo(turn, ds)
      onMessage(turn, ds, ?Notify) do (summary: string, attrs: Attrs):
        doAssert isInitted()
        let
          body = getOrDefault(attrs, Symbol"body")
          icon = getOrDefault(attrs, Symbol"icon")
          n = notification_new(summary, body, icon)

        case getOrDefault(attrs, Symbol"urgency").normalize
        of "high", "critical":
          set_urgency(n, Critical)
        of "low":
          set_urgency(n, Low)
        else: discard

        let category = getOrDefault(attrs, Symbol"category")
        if category != "":
          set_category(n, category)

        show(n)
  do:
    uninit()
